/*
 *  AFS - The native AtheOS file-system
 *  Copyright (C) 1999 - 2001 Kurt Skauen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*  Ported to BeOS by Ronny Schoebel, 2001
 *  Original Version: AtheOS 0.3.2
 */


#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <malloc.h>
#include <sys/types.h>
#include <KernelExport.h>
#include "afsbuild.h"
#include "macros.h"
#include "fsproto.h"
#include "afs.h"
#include "btree.h"

int afs_open_indexdir( void* pVolume, void** ppCookie )
{
    AfsFileCookie_s* psCookie;
  
	psCookie = calloc( 1, sizeof( AfsFileCookie_s ) );
  
    if ( NULL == psCookie )
    {
		return( ENOMEM );
    }
	psCookie->fc_nOMode = O_RDONLY;
	psCookie->fc_sBIterator.bi_nCurNode = -1;
  
	*ppCookie = psCookie;
    
	return( B_OK );
}

int afs_close_indexdir( void* pVolume, void* pCookie )
{
	return( B_OK );
}

int afs_free_indexdircookie( void *pVolume, void *pCookie )
{
	free( pCookie );
	return( B_OK );
}

int afs_rewind_indexdir( void* pVolume, void* pCookie )
{
	return( B_OK );
}


int afs_read_indexdir( void* pVolume, void* pCookie, long *pnNum, struct dirent* psBuffer, size_t nBufferSize )
{
	AfsVolume_s*		psVolume     = pVolume;
	AfsSuperBlock_s*	psSuperBlock = psVolume->av_psSuperBlock;
	AfsFileCookie_s*	psCookie     = pCookie;
	AfsInode_s*			psIndexDir;
	int					nError;
	
	*pnNum = 0;
	
	SEM_LOCK( psVolume->av_hIndexDirMutex );
	nError = afs_do_read_inode( psVolume, &psSuperBlock->as_sIndexDir, &psIndexDir );
	
	if ( nError < 0 )
	{
		DPRINTF( "Error: afs_create_index() failed to read index directory inode %d\n", nError );
		goto error;
	}
	nError = afs_do_read_dir( psVolume, psIndexDir, &psCookie->fc_sBIterator, -1, psBuffer, nBufferSize );
	if( nError < 0 )
	{
		*pnNum = 0;
	}
	else
	{
		*pnNum = nError;
		nError = B_OK;
	}
	free( psIndexDir );
error:
	SEM_UNLOCK( psVolume->av_hIndexDirMutex );
	return( nError );
}

int afs_create_index( void* pVolume, const char* pzName, int nType, int nFlags )
{
	AfsVolume_s*		psVolume     = pVolume;
	AfsSuperBlock_s*	psSuperBlock = psVolume->av_psSuperBlock;
	AfsInode_s*			psIndexDir;
	int					nError;
	int 				nNameLen;
	
	if(pzName == NULL)
	{
		return (EINVAL);
	}
	nNameLen = strlen(pzName);
	
	SEM_LOCK( psVolume->av_hIndexDirMutex );
	
	nError = afs_begin_transaction( psVolume );
	if ( nError < 0 )
	{
		DPRINTF( "PANIC : afs_create_index() failed to start transaction %d\n", nError );
		goto error1;
	}
	
	nError = afs_do_read_inode( psVolume, &psSuperBlock->as_sIndexDir, &psIndexDir );
	
	if ( nError < 0 )
	{
		DPRINTF( "Error: afs_create_index() failed to read index directory inode %d\n", nError );
		goto error2;
	}
	nError = afs_create_inode( psVolume, psIndexDir, 0777 | S_IFDIR, 0, 0, pzName, nNameLen, NULL );
	free( psIndexDir );
error2:
	afs_end_transaction( psVolume, true );
error1:
	SEM_UNLOCK( psVolume->av_hIndexDirMutex );
	
	return( nError );
}


int afs_remove_index( void* pVolume, const char* pzName)
{
    return( B_OK );
}

int afs_stat_index( void* pVolume, const char* pzName, struct index_info* psBuffer )
{
    return( B_OK );
}

