/*
 *  AFS for BeOS - The native AtheOS file-system for BeOS
 *  Copyright (C) 2001 Ronny Schoebel
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <SupportDefs.h>
#include <malloc.h>
#include <string.h>
#include <KernelExport.h>
#include <fs_info.h>
#include <scsi.h>
#include "afsbuild.h"
#include "lock.h"
#include "cache.h"
#include "support.h"

status_t lock_removable_device( int nDev, bool bState )
{
	return ioctl( nDev, B_SCSI_PREVENT_ALLOW, &bState, sizeof( bState ) );
}

status_t get_device_flags( int nDev, uint32* pnFlags )
{
	struct stat sDevStat;
	status_t nError;
	
	nError = fstat( nDev, &sDevStat );
	if( nError < B_OK )
	{
		return nError;
	}
	*pnFlags |= S_ISREG( sDevStat.st_mode ) ? B_FS_IS_REMOVABLE : 0;
	*pnFlags |= ( sDevStat.st_mode & S_IWUSR ) ? B_FS_IS_READONLY : 0;
	
	return nError;
}

int write_logged_blocks( int nDev, off_t nBlockNum, const void* pBuffer, uint nCount, int nBlockSize,
	void (*pFunc)(off_t nBlocknum, size_t nBlocks, void *pArg), void* pArg )
{
	void	*pBlock;
	int		i;
	int		nError;
	off_t	*pBlocks;

	pBlocks = (off_t *)malloc(sizeof(off_t) * nCount);
	if(pBlocks == NULL)
	{
		panic("write_logged_blocks(): couldn't get memory for block list!\n");
		return ENOMEM;
	}	
	for(i = 0; i < nCount; i++)
	{
		pBlock = get_empty_block(nDev, nBlockNum + i, nBlockSize);
		
		if(pBlock != NULL)
		{
			memcpy((char *)pBlock, ((char *)pBuffer) + i * nBlockSize, nBlockSize);
			pBlocks[i] = nBlockNum + i;
		}
		else
		{
			panic("write_logged_blocks() failed to find block %ld!\n", nBlockNum + i);
			return EIO;
		}
	}
	nError = set_blocks_info(nDev, pBlocks, nCount, pFunc, pArg);
	if(nError < 0)
	{
		free(pBlocks);
		panic("write_logged_blocks failed to rgister callback handlers! Error: %d\n", nError);
	}
	free(pBlocks);
	return B_OK;
}	
	
gid_t getfsgid(void)
{
	return getgid();
}

uid_t getfsuid(void)
{
	return getuid();
}
		 
	
